<?php
// $Id: forms.inc,v 1.2 2009/08/04 22:53:28 einsicht Exp $
/**
 * @file
 * QMO Bugzilla Dashboard - Forms
 * @author Paul Craciunoiu
 */

/*********************************************** DASHBOARD UI *******************************************/
/**
 * Callback function on rows or cols change
 * @see form_qmo_bugzilla_dash()
 */
function form_qmo_bugzilla_dash_resize($form, &$form_state) {
  $form_state['rebuild'] = TRUE;
}

/**
 * Callback function on weights update
 * @see form_qmo_bugzilla_dash()
 */
function form_qmo_bugzilla_dash_weight($form, &$form_state) {
  global $user;
  $reply = array();
  $chart_ids = explode(',', $form_state['values']['chart_ids']);
  if (!empty($chart_ids)) {
    foreach ($chart_ids as $id) {
      $query_w = "UPDATE {" . QMO_BUGZILLA_TABLE_MY_CHART . "} SET weight = %d  WHERE (chart_id = %d ) AND (user_id = %d) ";
      $result_w = db_query($query_w, $form_state['values']['chart_' . $id . '_weight_value'], $id, $user->uid);
      if (!$result_w) {
        $reply[10] = array('fail-weights-update');
      }
    }
  }
  set_messages($reply);
  $form_state['rebuild'] = TRUE;
}

function form_qmo_bugzilla_dash_goto_section($form, &$form_state) {
  $pagenum = $form_state['values']['pagenum'];
  $uid = $form_state['values']['uid'];
  drupal_goto('qmo_bugzilla/user/' . $uid . '/' . $pagenum, 'section=' . $form_state['values']['section_num']);
}

/**
 * @link qmo_bugzilla
 * @see qmo_bugzilla_dash()
 */
function form_qmo_bugzilla_dash($form_state, $pagenum, $uid) {
  global $user;
  $page_title = variable_get('qmo_bugzilla_my_page_' . $uid . '_' . $pagenum . '_title', t('Page ' . $pagenum));
  drupal_set_title($page_title);
  $reply = array();
  //timeplot init script:
  $timeplot = new QMOTimeplot();

  //constants
  $BUTTON_PREV = drupal_get_path('module', 'qmo_bugzilla') . '/img/prev.png';
  $BUTTON_NEXT = drupal_get_path('module', 'qmo_bugzilla') . '/img/next.png';

  $section_num = get_first_nonempty($_GET['section'], 1);

  $size_rows = get_first_nonempty($form_state['values']['size_rows'], variable_get('qmo_bugzilla_size_rows_' . $uid . '_' . $pagenum, SIZE_ROWS) );
  if ($form_state['values']['size_rows']) {
    variable_set('qmo_bugzilla_size_rows_' . $uid . '_' . $pagenum, $size_rows);
  }
  $size_cols = get_first_nonempty($form_state['values']['size_cols'], variable_get('qmo_bugzilla_size_cols_' . $uid . '_' . $pagenum, SIZE_COLS) );
  if ($form_state['values']['size_cols']) {
    variable_set('qmo_bugzilla_size_cols_' . $uid . '_' . $pagenum, $size_cols);
  }
  //total number of cells (different from $num_charts)
  $cells_num = $size_rows * $size_cols;
  $args = array($uid, $pagenum);
  //get sharing condition:
  global $user;
  $valid_roles = array_keys(user_roles(FALSE, QMO_BUGZILLA_ACCESS));
  $roles_to_check = $user->roles;
  $sharing_condition = qmo_bugzilla_build_sharing_condition($roles_to_check, $valid_roles, TRUE);
  $args = array_merge($args, $sharing_condition['args']);
  if (!empty($sharing_condition['query'])) {
    $sharing_condition = 'AND (' . $sharing_condition['query'] . ')';
  }
  $charts_count = db_result(db_query("SELECT COUNT(*) FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} JOIN {" . QMO_BUGZILLA_TABLE_CHART . "} ON ({" . QMO_BUGZILLA_TABLE_CHART . "}.id = {" . QMO_BUGZILLA_TABLE_MY_CHART . "}.chart_id) WHERE ({" . QMO_BUGZILLA_TABLE_MY_CHART . "}.user_id = %d) AND ({" . QMO_BUGZILLA_TABLE_MY_CHART . "}.pagenum = %d) " . $sharing_condition . " ORDER BY {" . QMO_BUGZILLA_TABLE_MY_CHART . "}.weight, {" . QMO_BUGZILLA_TABLE_CHART . "}.added DESC", $args));
  if (!$charts_count) $charts_count = 1;
  $section_count = (int)(1 + ($charts_count-1)/$cells_num);
  
  //which charts?
  $query = "SELECT * FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} JOIN {" . QMO_BUGZILLA_TABLE_CHART . "} ON ({" . QMO_BUGZILLA_TABLE_CHART . "}.id = {" . QMO_BUGZILLA_TABLE_MY_CHART . "}.chart_id) WHERE ({" . QMO_BUGZILLA_TABLE_MY_CHART . "}.user_id = %d) AND ({" . QMO_BUGZILLA_TABLE_MY_CHART . "}.pagenum = %d) " . $sharing_condition . " ORDER BY {" . QMO_BUGZILLA_TABLE_MY_CHART . "}.weight, {" . QMO_BUGZILLA_TABLE_CHART . "}.added DESC";
  
  $result = db_query_range($query, $args, ($section_num-1)*$cells_num, $section_num*$cells_num);
  if (!$result) {
  $reply[] = array('fail-charts');
  set_messages($reply);
  return FALSE;
  }
  else {
    $charts = array();
    while ($charts[] = db_fetch_array($result)) {
      //do what?
    }
    $num_charts = count($charts);
  }

  //various variables
  $href_prev = ($section_num-1) ? '?section=' . ($section_num-1) : '#';
  $href_next = ($section_num<$section_count) ? '?section=' . ($section_num+1) : '#';

  //now on to the form fields
  $form = array();
  $form['pagenum'] = array(
      '#type' => 'value',
      '#value' => $pagenum,
  );
  $form['uid'] = array(
      '#type' => 'value',
      '#value' => $uid,
  );

  $form['BUTTON_PREV'] = array(
    '#value' => '<table class="dash dash-top"><tr><td>
                 <a ' . drupal_attributes( array('href' => $href_prev, 'title' => t('Previous')) ) . '>
       <img ' . drupal_attributes( array('alt' => '<<', 'src' => $BUTTON_PREV) ) . '/>
       </a>
       </td>',
  );

  if ($uid == $user->uid) {
    $form['size_rows'] = array(
      '#type' => 'textfield',
      '#title' => t('Rows'),
      '#default_value' => $size_rows,
      '#size' => 2,
      '#maxlength' => 2,
      '#prefix' => '<td>',
    );
  
    $form['size_cols'] = array(
      '#type' => 'textfield',
      '#title' => t('Cols'),
      '#default_value' => $size_cols,
      '#size' => 2,
      '#maxlength' => 2,
    );
  
    $form['page_resize'] = array(
      '#type' => 'submit',
      '#value' => t('Change'),
      '#submit' => array('form_qmo_bugzilla_dash_resize'),
      '#suffix' => '</td>',
    );
  }

  $form['section_num'] = array(
    '#type' => 'textfield',
    '#title' => t('Section'),
    '#default_value' => $section_num,
    '#size' => 2,
    '#maxlength' => 2,
    '#prefix' => '<td>',
    '#suffix' => t(' of !section_count', array('!section_count' => $section_count) ),
  );

  $form['section_go'] = array(
    '#type' => 'submit',
    '#value' => t('Go'),
    '#submit' => array('form_qmo_bugzilla_dash_goto_section'),
    '#suffix' => '</td>',
  );

  if ($uid == $user->uid) {
    $form['update_weights'] = array(
      '#type' => 'submit',
      '#value' => t('Update weights'),
      '#submit' => array('form_qmo_bugzilla_dash_weight'),
      '#prefix' => '<td>',
      '#suffix' => '</td>',
    );
  }

  $form['BUTTON_NEXT'] = array(
    '#value' => '<td>
                 <a ' . drupal_attributes( array('href' => $href_next, 'title' => t('Next')) ) . '>
       <img ' . drupal_attributes( array('alt' => '>>', 'src' => $BUTTON_NEXT) ) . '/>
       </a>
       </td></tr></table>',
  );
  $chart_ids = "";
  for ($i=1; $i<=$size_rows; $i++) {
    for ($j=1; $j<=$size_cols; $j++) {
      if ($j==1) {
        $prefix = '<tr>';
      }
      elseif ($j==$size_cols) {
        $suffix = '</tr>';
      }
      $chart_dash_number = $size_cols*($i-1)+$j-1;
      $id = $charts[$chart_dash_number]['id'];
      $title = get_first_nonempty( $charts[$chart_dash_number]['title'], t('Chart !i', array( '!i' => ($chart_dash_number+1) )) );
      $link = (is_null($id)) ? NULL : "qmo_bugzilla/chart/" . $id;
      $fieldset_title = $link ? l($title, $link) : '<a href="#">' . $title . '</a>';
      $height = variable_get('qmo_bugzilla_dash_chart_height', CHART_HEIGHT_DASH_DEFAULT);
      if (!is_null($id)) {
        $chart_display = _build_chart($charts[$chart_dash_number], $timeplot, 'dash');
        $chart_ids .= $id . ',';
      }
      else {
        $chart_display = qmo_bugzilla_chart_display_empty($height);
      }

      $form['chart_' . $i . '_' . $j] = array(
        '#type' => 'fieldset',
        '#title' => $fieldset_title,
        '#prefix' => $prefix . '<td width="' . number_format(100/$size_cols, 1) . '%">',
        '#suffix' => '</td>' . $suffix,
      );
      if (!is_null($id)) {
        $prefix = '<ul class="chart-options">';
        $access_edit = FALSE;
        if (($chart['user_id'] == $user->uid) && (user_access(QMO_BUGZILLA_EDIT_OWN))) {
          $access_edit = TRUE;
        }
        elseif (qmo_check_sharing($chart['id']) && (user_access(QMO_BUGZILLA_EDIT_SHARED))) {
          $access_edit = TRUE;
        }
        elseif (user_access(QMO_BUGZILLA_EDIT_ANY)) {
          $access_edit = TRUE;
        }
        $access_run = FALSE;
        if (($chart['user_id'] == $user->uid) && (user_access(QMO_BUGZILLA_RUN_OWN))) {
          $access_run = TRUE;
        }
        elseif (user_access(QMO_BUGZILLA_RUN_ANY)) {
          $access_run = TRUE;
        }
        if ($access_edit) {
          $form['chart_' . $i . '_' . $j]['chart_' . $id . '_edit'] = array(
            '#value' => l(t('Edit'), 'qmo_bugzilla/chart/' . $id . '/edit'),
            '#prefix' => $prefix . '<li class="chart-edit">',
            '#suffix' => '</li>',
            );
          $prefix = "";
        }
        if ($access_run) {
          $form['chart_' . $i . '_' . $j]['chart_' . $id . '_run'] = array(
            '#value' => l(t('Run'), 'qmo_bugzilla/chart/' . $id . '/run'),
            '#prefix' => $prefix . '<li class="chart-run">',
            '#suffix' => '</li>',
            );
          $prefix = "";
        }
        $form['chart_' . $i . '_' . $j]['chart_' . $id . '_file'] = array(
          '#value' => l(t('Data'), 'qmo_bugzilla/chart/' . $id . '/data'),
          '#prefix' => $prefix . '<li class="chart-file">',
          '#suffix' => '</li>',
        );
        if ($access_edit) {
          $form['chart_' . $i . '_' . $j]['chart_' . $id . '_weight_value'] = array(
            '#type' => 'textfield',
            '#title' => t('W'),
            '#default_value' => $charts[$size_cols*($i-1)+$j-1]['weight'],
            '#size' => 2,
            '#maxlength' => 2,
            '#prefix' => '<li class="chart-weight-value inline">',
            '#suffix' => '</li></ul>',
          );
        }
        else {
          $form['chart_' . $i . '_' . $j]['chart_' . $id . '_list_end'] = array(
              '#value' => '</ul>',
          );
        }
      }

      $form['chart_' . $i . '_' . $j]['chart_' . $i . '_' . $j . '_data'] = array(
        '#value' => $chart_display,
      );


      $prefix='';
      $suffix='';
    }
  }
  $form['chart_ids'] = array(
      '#type' => 'value',
      '#value' => substr($chart_ids, 0, -1),
  );


  $form['chart_1_1']['#prefix'] = '<table><tr><td>';
  $form['chart_' . $size_rows . '_' . $size_cols]['#suffix'] = '</td></tr></table>';

  if (!is_null($timeplot)) {
    variable_set('qmo_bugzilla_timeplot_object', $timeplot);
  }
  return $form;
}

/**
 * Clone chart
 */
function form_qmo_bugzilla_chart_clone($form, &$form_state) {
  drupal_goto('qmo_bugzilla/chart/' . $form_state['values']['chart_id'] . '/clone');
}

/**
 * Add query to add/edit chart form
 */
function form_qmo_bugzilla_chart_query_add($form, &$form_state) {
  $form_state['values']['query_num']++;
  $reply = array( array('query-add') );
  $form_state['rebuild'] = TRUE;
  set_messages($reply);
}
/**
 * Remove query from add/edit chart form
 */
function form_qmo_bugzilla_chart_query_remove($form, &$form_state) {
  if ($form_state['values']['query_num'] > 1) {
    $form_state['values']['query_num']--;
    $reply = array( array('query-remove') );
  }
  else {
  $reply = array( array('fail-query-remove') );
  }
  $form_state['rebuild'] = TRUE;
  set_messages($reply);
}

/**
 * Configure pages for QMO Bugzilla
 */
function form_qmo_bugzilla_my_pages_manage($form_state) {
  global $user;
  $uid = $user->uid;
  $form = array();
  $pagecount = variable_get('qmo_bugzilla_my_pagecount_' . $uid, 1);
  $form['qmo_bugzilla_my_pagecount_' . $uid] = array(
      '#type' => 'textfield',
      '#title' => t('Number of pages'),
      '#description' => t('The number of pages on your dashboard. All are placed under qmo_bugzilla/user/uid/pagenum, where uid is your used id and pagenum is the page number.'),
      '#default_value' => $pagecount,
      '#size' => 5,
      '#maxlength' => 5,
      '#required' => TRUE,
  );
  
  $form['pages'] = array(
      '#type' => 'fieldset',
      '#title' => t('Pages'),
      '#collapsible' => TRUE,
  );
  for ($i=1; $i<=$pagecount; $i++) {
    $form['pages']['qmo_bugzilla_my_page_' . $uid . '_' . $i . '_title'] = array(
        '#type' => 'textfield',
        '#title' => t('Page !i title', array('!i' => $i)),
        '#default_value' => variable_get('qmo_bugzilla_my_page_' . $uid . '_' . $i . '_title', t('Page ' . $i)),
        '#size' => 30,
        '#maxlength' => 30,
        '#description' => t('The page title as seen on the dashboard page at !link', array('!link' => l('qmo_bugzilla/user/' . $uid . '/' . $i, 'qmo_bugzilla/user/' . $uid . '/' . $i))),
        '#required' => FALSE,
      );
    
  }
  return system_settings_form($form);
}

// ******************************* REMOVE CHART ********************************//
function form_qmo_bugzilla_chart_remove($form, &$form_state) {
  $id = $form_state['values']['chart_id'];
  qmo_bugzilla_chart_remove($id, $form_state['values']['title'], FALSE);
  drupal_goto('qmo_bugzilla');
}
function qmo_bugzilla_chart_remove($id, $title = NULL, $silent = TRUE) {
  global $user;
  $success = FALSE;
  if (!$silent) {
    $reply = array();
    $goto = variable_get('qmo_bugzilla_chart_edit_referer', DEFAULT_REDIRECT);
  }
  $result = db_query("SELECT user_id FROM {" . QMO_BUGZILLA_TABLE_CHART . "} WHERE id= %d ", $id);
  $user_id = db_result($result);
  if ($user_id == $user->uid) {
    $access = user_access(QMO_BUGZILLA_REMOVE_OWN);
  }
  else {
    $access = user_access(QMO_BUGZILLA_REMOVE_ANY);
  }
  if ($access) {
    //delete from the bottom up: results -> queries -> chart+timeplot+my
    //delete results
        qmo_bugzilla_chart_empty($id, FALSE, TRUE);
    //delete queries
        $query_q = "DELETE FROM {" . QMO_BUGZILLA_TABLE_QUERY . "} WHERE chart_id= %d ";
    $result = db_query($query_q, $id);
    //delete my chart info
        $query = "DELETE FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} WHERE chart_id = %d AND user_id = %d";
    $result = db_query($query, $id, $user->uid);
    //delete chart
        $query = "DELETE FROM {" . QMO_BUGZILLA_TABLE_CHART . "} WHERE id= %d ";
    $result = db_query($query, $id);
    //delete path alias
        $path = 'qmo_bugzilla/chart/'.$id;
    path_set_alias($path);

    if (!$silent) {
      $reply[] = array('remove-chart', array($title));
    }
    else {
      $success = TRUE;
    }
  }
  else {
    if (!$silent) {
      $reply[] = array('remove-chart-deny');
    }
    else {
      $success = FALSE;
    }
  }
  if (!$silent) {
    set_messages($reply);
    drupal_goto($goto);
  }
  else {
    return $success;
  }
}
// ******************************* ADDEDIT CHART ********************************//
function form_qmo_bugzilla_chart_addedit_submit($form, &$form_state) {
  global $user;
  $uid = $user->uid;
  //initialize variables
  $reply = array();
  $goto = variable_get('qmo_bugzilla_chart_edit_referer', DEFAULT_REDIRECT);
  $added_updated = date(DATE_SQL_FORMAT);
  $id = $form_state['values']['chart_id'];
  $old_id = $form_state['values']['old_id'];
  
  if (!is_null($id)) {
    //update existing
    $query = "UPDATE {" . QMO_BUGZILLA_TABLE_CHART . "} SET  title = '%s' , description = '%s' , type = %d , path = '%s' , start = '%s' , stop = '%s' , height = '%s' , width = '%s' , sharing = '%s' , updated = '%s' WHERE id = %d ";
    $reply[] = array('chart-edit', 'vars' => array($form_state['values']['title']) );
  }
  else {
    //add new, columns: " id , user_id , title , description , type , path , start , stop , height , width , sharing, updated , added "
        $query = "INSERT INTO {" . QMO_BUGZILLA_TABLE_CHART . "} VALUES ( NULL , %d , '%s' , '%s' , %d , '%s' , '%s' , '%s' , '%s' , '%s' , '%s' ,  '%s' ,  '%s' )";
    $reply[] = array('chart-add', 'vars' => array($form_state['values']['title']) );
  }
  $query_values = array(
    $form_state['values']['title'],
    $form_state['values']['description'],
    $form_state['values']['display_type'],
    $form_state['values']['path'],
    $form_state['values']['start'],
    $form_state['values']['stop'],
    $form_state['values']['height'],
    $form_state['values']['width'],
    format_sharing($form_state['values']['sharing']),
    $added_updated,
  );
  if (is_null($id)) {
    //for new charts, also insert the added date and user
    $query_values[] = $added_updated;
    array_unshift($query_values, $uid);
  }
  else {
    //for old charts update existing chart values
    $query_values[] = $id;
  }
  $result = db_query($query, $query_values);
  if (is_null($id)) {
    //get id of newly added chart
    $id = db_last_insert_id(QMO_BUGZILLA_TABLE_CHART, 'id');
    //mark this is a new chart for chart_queries update purposes
    $new = TRUE;
    if (is_null($id)) {
      //in the tiny case that the above will fail, stop right here
      $reply[] = array('fail-chart-id');
      set_messages($reply);
      return FALSE;
    }
  }
  $path = 'qmo_bugzilla/chart/'.$id;
  if ( $form_state['values']['path'] ) {
    $old_path = base_path() . drupal_get_path_alias($path);
    $old_goto = substr($goto, -strlen($old_path) );
    if ($old_goto == $old_path) {
      $goto = $form_state['values']['path'];
    }
    path_set_alias($path);
    path_set_alias($path, $form_state['values']['path']);
  }
  $new_chart = $form_state['values']['chart_new'];
  $add_to_my_charts = $form_state['values']['add_to_my'];
  if ($add_to_my_charts) {
    if ($new_chart) {
      //add it to MY_CHARTS (id, user_id, chart_id, weight)
      $query_my = "INSERT INTO {" . QMO_BUGZILLA_TABLE_MY_CHART . "} VALUES ( NULL, %d, %d, %d, %d) ";
      $query_my_args = array($user->uid, $id, $form_state['values']['weight'], $form_state['values']['pagenum']);
    }
    else {
      //update MY_CHARTS
      $query_my = "UPDATE {" . QMO_BUGZILLA_TABLE_MY_CHART . "} SET weight = %d, pagenum = %d WHERE (chart_id = %d ) AND (user_id = %d) ";
      $query_my_args = array($form_state['values']['weight'], $form_state['values']['pagenum'], $id, $user->uid);
    }
    $result_my = db_query($query_my, $query_my_args);
  }
  else {
    //remove from MY_CHARTS
    $query_my = "DELETE FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} WHERE (chart_id = %d ) AND (user_id = %d) ";
    $query_my_args = array($id, $user->uid);
    $result_my = db_query($query_my, $query_my_args);
  }
  
  //set up adding queries
  if ($result) {
    $q_num = count($form_state['values']['query']);
    $ids = array();
    //delete and prepare replace of existing queries, if any
    if (!$new) {
      $query_2 = "SELECT id FROM {" . QMO_BUGZILLA_TABLE_QUERY . "} WHERE chart_id = %d ORDER BY %s ";
      $result_2 = db_query($query_2, $id, 'id');
      while ($row_2 = db_fetch_object($result_2)) {
        $ids[] = $row_2->id;
      }
      $query_q = "DELETE FROM {" . QMO_BUGZILLA_TABLE_QUERY . "} WHERE chart_id= %d ";
      $result = db_query($query_q, $id);
    }

    if ($result) {
      $query_q = "INSERT INTO {" . QMO_BUGZILLA_TABLE_QUERY . "} VALUES ";
      //" id , chart_id , title , description, url , enabled, count_total , map_tag_values , mapping , step , nextrun , added "
      $query_q_values = array();
      for ($i=1; $i<=$q_num; $i++) {
        //set up the values
        $step = $form_state['values']['query'][$i]['q-step'];
        if (!is_null($ids[$i-1])) {
          $query_q_values[] = $ids[$i-1];
        }
        $query_q_values[] =   $id;
        $query_q_values[] =   $form_state['values']['query'][$i]['title'];
        $query_q_values[] =   $form_state['values']['query'][$i]['description'];
        $query_q_values[] =   $form_state['values']['query'][$i]['url'];
        $query_q_values[] =   ($form_state['values']['query'][$i]['enabled']) ? 1 : 0;
        $query_q_values[] =   ($form_state['values']['query'][$i]['count_total']) ? 1 : 0;
        $query_q_values[] =   $form_state['values']['query'][$i]['map_tag_values'] ? 1 : 0;
        $query_q_values[] =   $form_state['values']['query'][$i]['unmatched_option'];
        $query_q_values[] =   $form_state['values']['query'][$i]['mapping'];
        $query_q_values[] =   $step;
        $query_q_values[] =   date(DATE_SQL_FORMAT, add_dates($added_updated, $step));
        $query_q_values[] =   $added_updated;
        if (is_null($ids[$i-1])) {
          $query_q .= "(
                      NULL , %d , '%s' , '%s' , '%s' , %d , %d , %d , %d , '%s' , '%s' , '%s' , '%s'
                      ),";
        }
        else {
          $query_q .= "(
                    %d , %d , '%s' , '%s' , '%s' , %d , %d , %d , %d, '%s' , '%s' , '%s' , '%s'
                      ),";
        }
      }
      $result = db_query(substr($query_q, 0, -1), $query_q_values);
      if (!$result) {
        $reply[] = array('fail-query-update');
      }
    }
    else {
        $reply[] = array('fail-query-delete');
    }
  }
  else {
    if (!is_null($id)) {
      $reply[] = array('fail-chart-update');
    }
    else {
      $reply[] = array('fail-chart-add');
    }
  }
  set_messages($reply);
  $form_state['redirect'] = $goto;
}

function form_qmo_bugzilla_chart_addedit(&$form_state, $id = NULL, $clone = FALSE) {
  global $user;
  $uid = $user->uid;
  if (!is_null($id) && !qmo_bugzilla_chart_exists($id, FALSE)) {
    return FALSE;
  }
  $new_chart = 1;
  if (!is_null($id)) {
      $query = "SELECT * FROM {" . QMO_BUGZILLA_TABLE_CHART . "} WHERE id = %d ";
      $result = db_query($query, $id);
      if ($chart = db_fetch_array($result)) {
        if ($clone) {
          $chart['title'] = t("!title (clone)", array('!title' => $chart['title']));
          drupal_set_title( t("Create chart %title", array('%title' => $chart['title'])) );
        }
        else {
          drupal_set_title( t("Edit chart %title", array('%title' => $chart['title'])) );
        }
        $query_2 = "SELECT * FROM {" . QMO_BUGZILLA_TABLE_QUERY . "} WHERE chart_id = %d ORDER BY %s";
        $result_2 = db_query($query_2, $id, 'id');
        $q_count = 0;
        while ($row_2 = db_fetch_array($result_2)) {
          $q_count++;
          $chart['q'][$q_count] = $row_2;
        }
        $chart['query_num'] = $q_count;
        
        //get the weight
        $query_my = "SELECT * FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} WHERE chart_id = %d AND user_id = %d ";
        $result_my = db_query($query_my, $id, $uid);
        $row_my = db_fetch_object($result_my);
        $chart['weight'] = ($row_my) ? $row_my->weight : 0;
        $chart['pagenum'] = ($row_my) ? $row_my->pagenum : 1;
        $new_chart = ($row_my) ? 0 : 1;
      }
      else {
        return t("Error: chart not found. ");
      }
      qmo_bugzilla_access_chart_edit($chart);
  }
  else {
    drupal_set_title( t("Add a new chart"));
  }
  $form = array();

  $q_num = get_first_nonempty($form_state['values']['query_num'], $chart['query_num'], 1);

  $form['query_num'] = array(
      '#type' => 'value',
      '#value' => $q_num,
  );
  if ($clone) {
    $form['old_id'] = array(
      '#type' => 'value',
      '#value' => $id,
    );
    $id = NULL;
    $new_chart = 1;
  }
  $form['chart_id'] = array(
      '#type' => 'value',
      '#value' => $id,
  );
  $form['chart_new'] = array(
      '#type' => 'value',
      '#value' => $new_chart,
  );

  if ($_SERVER['HTTP_REFERER'] != selfurl()) {
    variable_set('qmo_bugzilla_chart_edit_referer', $_SERVER['HTTP_REFERER']);
  }
  
  $form['basic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Basic setup'),
    '#collapsible' => TRUE,
  );
  if ($clone) {
    $form['basic']['#description'] = t("Note that cloning a chart does not clone the chart's data.");
  }
  $form['basic']['display_type'] = array(
    '#type' => 'select',
    '#title' => t('Type'),
    '#default_value' => get_first_nonempty($form_state['values']['display_type'], $chart['type'], 0),
    '#options' => array( 0 => t('Bar chart'), 1 => t('Pie chart'), -1 => t('Timeplot')),
    '#prefix' => '<span class="inline">',
    '#suffix' => '</span>',
  );
  $form['basic']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => get_first_nonempty($form_state['values']['title'], $chart['title'], ''),
    '#maxlength' => 255,
    '#size' => 35,
    '#prefix' => '<span class="inline chart-edit-title">',
    '#suffix' => '</span>',
  );
  $form['basic']['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => get_first_nonempty($form_state['values']['description'], $chart['description'], ''),
    '#description' => t('Describe what this chart will show.'),
    '#required' => FALSE,
  );
  $form['basic']['path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path (Alias)'),
    '#default_value' => get_first_nonempty($form_state['values']['path'], $chart['path'], ''),
    //'#description' => t('If you want a more descriptive URL for viewing the chart, include it here.'),
    '#maxlength' => 255,
    '#size' => 20,
    '#prefix' => '<span class="inline">',
    '#suffix' => '</span>',
                                );
  $form['basic']['height'] = array(
    '#type' => 'textfield',
    '#title' => t('Height'),
    '#default_value' => get_first_nonempty( $form_state['values']['height'], $chart['height'], variable_get('qmo_bugzilla_height_default', CHART_HEIGHT_DEFAULT) ),
    //'#description' => t('Use pixels (px) or percentages (%). This will be added to the CSS of the chart.'),
    '#size' => 4,
    '#maxlength' => 6,
    '#prefix' => '<span class="inline chart-edit-height">',
    '#suffix' => '</span>',

  );
  $form['basic']['width'] = array(
    '#type' => 'textfield',
    '#title' => t('Width'),
    '#default_value' => get_first_nonempty( $form_state['values']['width'], $chart['width'], CHART_WIDTH_DEFAULT),
    //'#description' => t('Use pixels (px) or percentages (%). This will be added to the CSS of the chart.'),
    '#size' => 4,
    '#maxlength' => 6,
    '#prefix' => '<span class="inline">',
    '#suffix' => '</span>',
  );
  //ask to add to my charts if not in there and I didn't create it:
  if (is_null($id) || ($chart['user_id'] == $uid)) {
    $add_to_my_disabled = TRUE;
    $add_to_my_value = TRUE;
  }
  else {
    $add_to_my_value = empty($chart['weight']) ? FALSE : TRUE;
  }
  $form['dash'] = array(
    '#type' => 'fieldset',
    '#title' => t('My Dashboard settings'),
    '#collapsible' => TRUE,
    '#description' => ($add_to_my_disabled) ? t('Own charts must be added to "My Charts"') : t('Add this chart to "My Charts"'),
  );
  $form['dash']['weight'] = array(
    '#type' => 'textfield',
    '#title' => t('Weight'),
    '#default_value' => get_first_nonempty( $form_state['values']['weight'], $chart['weight'], 0),
    '#size' => 2,
    '#maxlength' => 2,
    '#prefix' => '<span class="inline">',
    '#suffix' => '</span>',
  );
  $form['dash']['add_to_my'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add to my charts'),
    '#default_value' => get_first_nonempty($form_state['values']['add_to_my'], $add_to_my_value, TRUE),
    '#disabled' => $add_to_my_disabled,
    '#prefix' => '<span class="inline">',
    '#suffix' => '</span>',
  );
  if ($add_to_my_disabled) {
    $form['dash']['add_to_my']['#value'] = TRUE;
  }
  $pagecount = variable_get('qmo_bugzilla_my_pagecount_' . $uid, 1);
  for ($i=1; $i<=$pagecount; $i++) {
    $pages[$i] = variable_get('qmo_bugzilla_my_page_' . $uid . '_' . $i . '_title', t('Page ' . $i));
  }
  $form['dash']['pagenum'] = array(
    '#type' => 'select',
    '#title' => t('Page'),
    '#default_value' => get_first_nonempty($form_state['values']['pagenum'], $chart['pagenum'], 1),
    '#options' => $pages,
    '#prefix' => '<span class="inline chart-addedit-pagenum">',
    '#suffix' => '</span>',
  );
  $form['sharing'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sharing'),
    '#tree' => TRUE,
    '#description' => t('Share this chart with any of the following user roles (leave blank to keep it completely private):'),
    '#collapsible' => TRUE,
  );
  $chart['sharing'] = unformat_sharing($chart['sharing']);
  $roles = user_roles(FALSE, QMO_BUGZILLA_ACCESS);
  foreach ($roles as $rid => $role) {
    $form['sharing'][$rid] = array(
      '#type' => 'checkbox',
      '#title' => $role,
      '#default_value' => get_first_nonempty($form_state['values']['sharing'][$rid], $chart['sharing'][$rid], 0),
    );
  }
  //timeplot
  // ******************************* TIMEPLOT CHART ********************************//
  //if ( ($form_state['values']['display_type'] == CHART_TYPE_TIMEPLOT) || ($chart['type'] == CHART_TYPE_TIMEPLOT) ) {
  $form['interval'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#title' => t('Active time interval'),
  );
  $form['interval']['start'] = array(
    '#type' => 'textfield',
    '#title' => t('Start running queries after'),
    '#default_value' => get_first_nonempty($form_state['values']['start'], $chart['start'], DEFAULT_FREQUENCY),
    '#description' => t('Format: YYYY-MM-dd hh:mm:ss'),
    '#size' => 50,
    '#maxlength' => 50,
  );
  $form['interval']['stop'] = array(
    '#type' => 'textfield',
    '#title' => t('Stop running queries before'),
    '#default_value' => get_first_nonempty($form_state['values']['stop'], $chart['stop'], DEFAULT_FREQUENCY),
    '#description' => t('Format: YYYY-MM-dd hh:mm:ss'),
    '#size' => 50,
    '#maxlength' => 50,
  );
  //}
  //not timeplot
  // ******************************* COMPARISON CHART ********************************//
  //else {
  //}

    // Add a wrapper for the queries and add/remove buttons.
    $form['queries_wrapper'] = array(
      '#type' => 'fieldset',
      '#title' => t('Queries'),
      '#tree' => FALSE,
      '#collapsible' => TRUE,
    );
    // Container for just the queries.
    $form['queries_wrapper']['query'] = array(
      '#prefix' => '<div id="queries">',
      '#suffix' => '</div>',
      '#tree' => TRUE,
    );
    // Add the current queries to the form.
    for ($i = 1; $i <= $q_num; $i++) {
      $form['queries_wrapper']['query'][$i] = form_qmo_bugzilla_chart_query_add_more($i, $chart['q'][$i]);
    }
    $form['queries_wrapper']['alert'] = array(
      '#value' => '',
      '#prefix' => '<div id="message"><div class="messages error">',
      '#suffix' => '</div></div>'
    );
  if (!is_null($id)) {
    //edit chart
    $form['submit'] = array('#type' => 'submit', '#value' => t('Update'));
    $access_string = ($user->uid == $chart['user_id']) ? QMO_BUGZILLA_REMOVE_OWN : QMO_BUGZILLA_REMOVE_ANY;
    if (user_access($access_string)) {
      $form['remove'] = array(
        '#type' => 'submit',
        '#value' => t('Remove'),
        '#submit' => array('form_qmo_bugzilla_chart_remove'),
      );
    }
    $form['clone'] = array(
    '#type' => 'submit',
    '#value' => t('Clone'),
    '#submit' => array('form_qmo_bugzilla_chart_clone'),
    );
  }
  else {
    //new chart
    $form['submit'] = array('#type' => 'submit', '#value' => t('Create new chart'));
  }
  // We name our button 'query_more' to avoid conflicts with other modules using
  // AHAH-enabled buttons with the id 'more' .
  $form['query_remove'] = array(
    '#type' => 'submit',
    '#value' => t('Remove last query'),
    '#weight' => 1,
    '#submit' => array('form_qmo_bugzilla_chart_query_remove'), // If no javascript action.
    '#ahah' => array(
      'path' => 'qmo_bugzilla/js/remove',
      'wrapper' => 'queries',
      'method' => 'replace',
      'effect' => 'none',
    ),
  );
  $form['query_more'] = array(
    '#type' => 'submit',
    '#value' => t('Add query'),
    '#weight' => 1,
    '#submit' => array('form_qmo_bugzilla_chart_query_add'), // If no javascript action.
    '#ahah' => array(
      'path' => 'qmo_bugzilla/js/add',
      'wrapper' => 'queries',
      'method' => 'replace',
      'effect' => 'none',
    ),
  );
  return $form;
}

/**
 * Menu callback for AHAH additions.
 * @see form_qmo_bugzilla_chart_addedit()
 */
function form_qmo_bugzilla_chart_query_add_js() {
  if (!$_POST['query']) {
    $i = 1;
  }
  else {
    $i = count($_POST['query'])+1;
  }

  // Build our new form element.
  $form_element = form_qmo_bugzilla_chart_query_add_more($i);
  drupal_alter('form', $form_element, array(), 'form_qmo_bugzilla_chart_query_add_js');

  // Build the new form.
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  // Add the new element to the stored form. Without adding the element to the
  // form, Drupal is not aware of this new elements existence and will not
  // process it. We retreive the cached form, add the element, and resave.
  $form = form_get_cache($form_build_id, $form_state);
  $form['queries_wrapper']['query'][$i] = $form_element;
  form_set_cache($form_build_id, $form, $form_state);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );
  // Rebuild the form.
  $form = form_builder('form_qmo_bugzilla_chart_addedit', $form, $form_state);

  // Render the new output.
  $query_form = $form['queries_wrapper']['query'];
  //remember old values:
  for ($j=1; $j<$i-1; $j++) {
    $q = $_POST['query'][$j];
    foreach ($q as $key => $value) {
      $query_form[$j][$key]['#value'] = $value;
    }
  }
  //set default values:
  $query_form[$i]['enabled']['#value'] = TRUE;
  $query_form[$i]['title']['#value'] = t('Query !i', array('!i' => $i));
  $query_form[$i]['url']['#value'] = DEFAULT_QUERY_URL;
  $query_form[$i]['q-step']['#value'] = DEFAULT_FREQUENCY;
  $query_form[$i]['enabled']['#value'] = TRUE;
  $query_form[$i]['enabled']['#value'] = TRUE;
  unset($query_form['#prefix'], $query_form['#suffix']); // Prevent duplicate wrappers.
  $output = theme('status_messages') . drupal_render($query_form);
  drupal_json(array('status' => TRUE, 'data' => $output));
}
/**
 * Menu callback for AHAH additions.
 * @see form_qmo_bugzilla_chart_addedit()
 */
function form_qmo_bugzilla_chart_query_remove_js() {
  $i = count($_POST['query']);

  // Build the new form.
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  // Add the new element to the stored form. Without adding the element to the
  // form, Drupal is not aware of this new elements existence and will not
  // process it. We retreive the cached form, add the element, and resave.
  $form = form_get_cache($form_build_id, $form_state);
  if ($i > 1) {
    unset($form['queries_wrapper']['query'][$i]);
    $form['queries_wrapper']['alert']['#value'] = '';
  }
  else {
    $form['queries_wrapper']['alert']['#value'] = t('Chart must have at least one query. ');
  }
  form_set_cache($form_build_id, $form, $form_state);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );
  // Rebuild the form.
  $form = form_builder('form_qmo_bugzilla_chart_addedit', $form, $form_state);
  // Render the new output.
  $query_form = $form['queries_wrapper']['query'];
  //remember old values:
  for ($j=1; $j<$i; $j++) {
    $q = $_POST['query'][$j];
    foreach ($q as $key => $value) {
      $query_form[$j][$key]['#value'] = $value;
    }
  }
  unset($query_form['#prefix'], $query_form['#suffix']); // Prevent duplicate wrappers.
  $output = theme('status_messages') . drupal_render($query_form) . drupal_render($form['queries_wrapper']['alert']);
  drupal_json(array('status' => TRUE, 'data' => $output));
}

function form_qmo_bugzilla_chart_query_add_more($i, $q = NULL, &$form_state = NULL) {
    $form = array(
      '#type' => 'fieldset',
      '#title' => t('Query !i', array('!i' => $i)),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    $form['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enabled'),
      '#description' => t('Uncheck to disable running and showing this query.'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['enabled'], $q['enabled'], TRUE),
      '#required' => FALSE,
      '#prefix' => '<div class="fltrt">',
      '#suffix' => '</div>',
    );
    $form['count_total'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show total'),
      '#description' => t('Include total count in charts.'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['count_total'], $q['count_total'], TRUE),
      '#required' => FALSE,
      '#prefix' => '<div class="fltrt">',
      '#suffix' => '</div>',
    );
    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['title'], $q['title'], t('Query ' . $i) ),
      '#description' => t('The title of this query. '),
      '#size' => 50,
      '#maxlength' => 1000,
    );
    $form['description'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['description'], $q['description'], ''),
      '#description' => t('Describe what this query will fetch, etc.'),
      '#required' => FALSE,
    );
    $form['url'] = array(
      '#type' => 'textfield',
      '#title' => t('URL'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['url'], $q['url'], DEFAULT_QUERY_URL),
      '#size' => 50,
      '#maxlength' => 5000,
    );
    $form['q-step'] = array(
      '#type' => 'textfield',
      '#title' => t('Frequency'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['q-step'], $q['step'], DEFAULT_FREQUENCY),
      '#description' => t('How often this query should be executed. Format: YYYY-MM-dd hh:mm:ss. Defaults to daily.'),
      '#size' => 19,
      '#maxlength' => 19,
    );
    $form['unmatched_option'] = array(
      '#type' => 'select',
      '#title' => t('Unmatched values'),
      '#description' => t('What to do with unmatched values?'),
      '#options' => get_value("UNMATCHED"),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['unmatched_option'], $q['unmatched_option'], 0),
      '#required' => FALSE,
      '#prefix' => '<div class="fltrt">',
      '#suffix' => '</div>',
    );
    $form['map_tag_values'] = array(
      '#type' => 'checkbox',
      '#title' => t('Map tag values'),
      '#description' => t('Create a mapping for values inside tags. Use the textarea below to map values.<br/>
                     <b>Warning</b>: This mapping displays aditional counts in the chart. Uncheck this to disable mapping. '),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['map_tag_values'], $q['map_tag_values'], FALSE),
      '#required' => FALSE,
    );
    $form['mapping'] = array(
      '#type' => 'textarea',
      '#title' => t('Mapping tags'),
      '#default_value' => get_first_nonempty($form_state['values']['query'][$i]['mapping'], $q['mapping']),
      '#description' => t('Helpful instructions:<ul>
                     <li>First row: TAGNAME_1|TAGNAME_2|..|TAGNAME_N</li>
                     <li>Every subsequent row: TAGNAME_1_VALUE|TAGNAME_2_VALUE|..|TAGNAME_N_VALUE|LABEL</li>
                     <li>So, every subsequent row has one more column than the first row</li>
                     <li><strong>Note: </strong>Very complex mapping may be slow, <a href="#">read about optimizing mapping</a>.</li>
                     </ul>'),
      '#required' => FALSE,
    );
  return $form;
}

/*********************************************** MANAGE CHARTS UI *******************************************/
function qmo_bugzilla_manage_header($has_charts) {
  $select_header = $has_charts ? theme('table_select_header_cell') : '';
  $header = array(
              $select_header,
              //array('data' => t('Edit')),
              array('data' => t('Type'), 'field' => 'type'),
              array('data' => t('Owner')),
              array('data' => t('Page')),
              array('data' => t('Title'), 'field' => 'title' ),
              array('data' => t('Path'), 'field' => 'path'),
              array('data' => t('Sharing'), 'field' => 'sharing'),
              array('data' => t('Updated'), 'field' => 'updated'),
              array('data' => t('Created'), 'sort' => 'desc', 'field' => 'added'),
              array('data' => t('Data')),
            );
  return $header;
}
/**
 * Theme node administration overview.
 *
 * @ingroup themeable
 */
function theme_form_qmo_bugzilla_charts_manage($form) {
  global $user;
  $has_charts = isset($form['title']) && is_array($form['title']);
  $header = qmo_bugzilla_manage_header($has_charts);
  $output = '';
  
  $output .= drupal_render($form['filters']);
  $output .= drupal_render($form['options']);
  if ($has_charts) {
    foreach (element_children($form['title']) as $key) {
      //format data properly first
      $form['updated'][$key]['#value'] = t( '!time ago', array('!time' => format_interval(time() - strtotime($form['updated'][$key]['#value']))) );
      $form['added'][$key]['#value'] = t( '!time ago', array('!time' => format_interval(time() - strtotime($form['added'][$key]['#value']))) );
      $form['user'][$key]['#value'] = user_load($form['user'][$key]['#value'])->name;
      $form['pagenum'][$key]['#value'] = variable_get('qmo_bugzilla_my_page_' . $user->uid . '_' . $form['pagenum'][$key]['#value'] . '_title', t('Page ' . $form['pagenum'][$key]['#value']));
      //adjust the sharing a little bit
      $form['sharing'][$key]['#value'] = implode(', ', qmo_bugzilla_get_string_roles($form['sharing'][$key]['#value']));
      //adjust the path a little bit
      if (!$form['path'][$key]['#value']) {
        $title_link = 'qmo_bugzilla/chart/' . $form['chart_ids'][$key]['#return_value'];
        $path = l( t('(no alias) '), $title_link );
      }
      else {
        $title_link = $form['path'][$key]['#value'];
        $path = l( $form['path'][$key]['#value'], $title_link );
      }
      $form['title'][$key]['#value'] = l($form['title'][$key]['#value'], $title_link);
      $form['path'][$key]['#value'] = $path;
      //adjust the data a little bit
      if ($form['data'][$key]['#value']) {
        /*if ($form['type'][$key]['#value'] == CHART_TYPE_TIMEPLOT) {
          $drupalpath = variable_get('qmo_bugzilla_chart_drupalpath', '');
          if (empty($drupalpath)) {
            $reply[] = array('missing-drupalpath');
            set_messages($reply);
            return FALSE;
          }
          $fullpath = variable_get('qmo_bugzilla_chart_fullpath', '');
          if (empty($fullpath)) {
            $reply[] = array('missing-fullpath');
            set_messages($reply);
            return FALSE;
          }
          $timestamp = variable_get('qmo_bugzilla_timestamp_' . $form['chart_ids'][$key]['#return_value'], '');
          $data_file = DIV . $form['chart_ids'][$key]['#return_value'] . '-' . $timestamp . EXT_COUNTS;
          if (file_exists($fullpath . $data_file)) {
            $data = '<span class="chart-data">' . l(t('Download'), $drupalpath . $data_file) . '</span>';
          }
          else {
            $reply[10] = array('no-datafile');
          }
        }
        else {
          $data = l(t('View'), "qmo_bugzilla/chart/" . $form['chart_ids'][$key]['#return_value'] . "/data");
        }*/
        $data = l(t('View'), "qmo_bugzilla/chart/" . $form['chart_ids'][$key]['#return_value'] . "/data");
      }
      else {
        $data = '<span class="chart-no-data">' . t('no data') . '</span>';
      }
      //for others, show a different data file
      $form['data'][$key]['#value'] = $data;
      
      $form['type'][$key]['#value'] = get_value('type:' . $form['type'][$key]['#value']);

      //then add it to the table
      $row = array();
      $row[] = drupal_render($form['chart_ids'][$key]);
      $row[] = drupal_render($form['type'][$key]);
      $row[] = drupal_render($form['user'][$key]);
      $row[] = drupal_render($form['pagenum'][$key]);
      $edit = $form['edit'][$key]['#value'] ? l( t('(edit)'), 'qmo_bugzilla/chart/' . $form['chart_ids'][$key]['#return_value'] . '/edit') : '';
      $form['title'][$key]['#value'] = $edit . ' ' . $form['title'][$key]['#value'];
      $row[] = drupal_render($form['title'][$key]);
      $row[] = drupal_render($form['path'][$key]);
      $row[] = drupal_render($form['sharing'][$key]);
      $row[] = drupal_render($form['updated'][$key]);
      $row[] = drupal_render($form['added'][$key]);
      $row[] = drupal_render($form['data'][$key]);
      $rows[] = $row;
    }
  }
  else {
    $rows[] = array(array('data' => t('No charts available.'), 'colspan' => '6'));
  }
  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }
  return $output;
}

function qmo_bugzilla_manage_filter_query() {
  $query = "";
  $args = array();
  $selects = array('user_id' => $_GET['users'], 'type' => $_GET['type'] ); //$_GET['title-option'], $_GET['description-option'], $_GET['path-alias'], $_GET['path-option']);
  foreach ($selects as $column => $options) {
    $filter_by_column = qmo_bugzilla_build_query_from_array($options, $column, QMO_BUGZILLA_TABLE_CHART);
    if (!empty($filter_by_column)) {
      $query .= $filter_by_column . ' AND ';
      $args = array_merge($args, $options);
    }
  }
  $text['title'] = array('text' => $_GET['title'], 'option' => $_GET['title-option'] );
  $text['path'] = array('text' => $_GET['path'], 'option' => $_GET['path-option']);
  
  foreach ($text as $field => $search) {
    $q = qmo_bugzilla_query_match_textfield($field, $search);
    if (!empty($q['query'])) {
      $query .= $q['query'];
      $args = empty($q['args']) ? NULL : array_merge($args, $q['args']);
    }
  }
  $pages = $_GET['pagenum'];
  $remove_and = FALSE;
  if (!empty($pages)) {
    $filter_by_column = qmo_bugzilla_build_query_from_array($pages, 'pagenum', QMO_BUGZILLA_TABLE_MY_CHART);
    $remove_and = TRUE;
    $query .= $filter_by_column . ' AND ';
    $args = array_merge($args, $pages);
  }
  if (!empty($query)) {
    $query = substr($query, 0, -4);
  }
  return array('query' => $query, 'args' => $args);
}

function qmo_bugzilla_query_match_textfield($field, $search) {
  $query = "";
  $args = array();
  if (!empty($search['text'])) {
    $not = "";
    if ($search['option'] == 0) {
      $not = "((path IS NULL) OR NOT";
      $not_par = ')';
    }
    $query .=  $not . "( LOWER(" . $field . ") LIKE LOWER('%%%s%') )" . $not_par . " AND ";
    $args[] = $search['text'];
  }
  return array('query' => $query, 'args' => $args);
}
  
function form_qmo_bugzilla_charts_manage_submit($form, &$form_state) {
  global $user;
  $reply = array();
  $op = $form_state['values']['operation'];
  //$charts = $form_state['values']['charts'];
  //print_r($charts);
  //print_r($form_state);
  switch ($op) {
    //change type
    case -1:
    case 0:
    case 1:
      $query_my_args = $form_state['values']['charts'];
      $ids_string = '';
      foreach ($query_my_args as $id) {
        $ids_string .= '%d,';
      }
      $ids_string = substr($ids_string, 0, -1);
      //remove from MY_CHARTS
      $query_my = "UPDATE {" . QMO_BUGZILLA_TABLE_CHART . "} SET type = " . $op . " WHERE id IN ( " . $ids_string . " )";
      $result_my = db_query($query_my, $query_my_args);
    case 5:
      $ids = $form_state['values']['charts'];
      foreach ($ids as $id) {
        qmo_bugzilla_chart_empty($id, FALSE, TRUE);
      }
      break;
    case 6:
      $ids = $form_state['values']['charts'];
      foreach ($ids as $id) {
        qmo_bugzilla_chart_remove($id);
      }
      break;
    //share & make private
    case 8:
      $sharing = format_sharing($form_state['values']['sharing']);
      $query_my_args = $form_state['values']['charts'];
      $ids_string = '';
      foreach ($query_my_args as $id) {
        $ids_string .= '%d,';
      }
      $ids_string = substr($ids_string, 0, -1);
      $query_my = "UPDATE {" . QMO_BUGZILLA_TABLE_CHART . "} SET sharing = '" . $sharing . "' WHERE id IN ( " . $ids_string . " )";
      $result_my = db_query($query_my, $query_my_args);
      break;
    case 9:
      $query_my_args = $form_state['values']['charts'];
      $ids_string = '';
      foreach ($query_my_args as $id) {
        $ids_string .= '%d,';
      }
      $ids_string = substr($ids_string, 0, -1);
      $query_my = "UPDATE {" . QMO_BUGZILLA_TABLE_CHART . "} SET path = '' WHERE id IN ( " . $ids_string . " )";
      $result_my = db_query($query_my, $query_my_args);
      foreach ($query_my_args as $id) {
        path_set_alias('qmo_bugzilla/chart/' . $id);
      }
      break;
    case 10:
      $chart_weights = $form_state['values']['chart_weights'];
      $new_charts = $form_state['values']['new_charts'];
      if (!empty($chart_weights)) {
        foreach ($chart_weights as $id => $weight) {
          if ($new_charts[$id]) {
            //add it to MY_CHARTS (id, user_id, chart_id, weight)
            $query_my = "INSERT INTO {" . QMO_BUGZILLA_TABLE_MY_CHART . "} VALUES ( NULL, %d, %d, %d) ";
            $query_my_args = array($user->uid, $id, $weight);
          }
          else {
            //update MY_CHARTS
            $query_my = "UPDATE {" . QMO_BUGZILLA_TABLE_MY_CHART . "} SET weight = %d  WHERE (chart_id = %d ) AND (user_id = %d) ";
            $query_my_args = array($weight, $id, $user->uid);
          }
          $result_my = db_query($query_my, $query_my_args);
          if (!$result_my) {
            $reply[10] = array('fail-weights-update');
          }
        }
      }
      set_messages($reply);
      break;
    case 11:
      $query_my_args = $form_state['values']['charts'];
      $ids_string = '';
      foreach ($query_my_args as $id) {
        $ids_string .= '%d,';
      }
      $ids_string = substr($ids_string, 0, -1);
      //remove from MY_CHARTS
      $query_my = "DELETE FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} WHERE chart_id IN ( " . $ids_string . " )";
      $result_my = db_query($query_my, $query_my_args);
      break;
  }
}

function form_qmo_bugzilla_charts_manage($form_state) {
  $op = $_GET['op'];
  $reply = array();
  
  //get some chart info
  $where = "";
  $query_args = array();
  $sharing_condition = "";
  //admin? if not & no filtering, check sharing.
  $filter_sharing = $_GET['sharing'];
  if (!empty($filter_sharing) || !user_access(QMO_BUGZILLA_ADMINISTER)) {
    global $user;
    $where = " WHERE ";
    $valid_roles = array_keys(user_roles(FALSE, QMO_BUGZILLA_ACCESS));
    $include_owner_charts = " ( {" . QMO_BUGZILLA_TABLE_CHART . "}.user_id = " . $user->uid . " )";
    if (!empty($filter_sharing)) {
      $include_owner_charts = "";
      $roles_to_check = array();
      foreach ($filter_sharing as $role) {
        $roles_to_check[$role] = $role;
      }
    }
    else {
      $roles_to_check = $user->roles;
    }
    $sharing_condition = qmo_bugzilla_build_sharing_condition($roles_to_check, $valid_roles);
    $query_args = array_merge($query_args, $sharing_condition['args']);
    $sharing_condition = $sharing_condition['query'];
    if ($include_owner_charts) {
      if ($sharing_condition) {
        $sharing_condition = '(' . $sharing_condition . ') OR ' . $include_owner_charts;
      }
      else {
        $sharing_condition = $include_owner_charts;
      }
    }
    //echo $sharing_condition;
  }

  $filter_query = qmo_bugzilla_manage_filter_query();
  $query_args = array_merge($query_args, $filter_query['args']);
  $filter_query = $filter_query['query'];
  if (!empty($filter_query)) {
    $where = " WHERE ";
  }
  if (!empty($sharing_condition) && !empty($filter_query)) {
    $filter_query = ' AND ' . $filter_query;
  }
  //echo $filter_query;
  //print_r($query_args);
  $query = "SELECT {" . QMO_BUGZILLA_TABLE_CHART . "}.id, title, {" . QMO_BUGZILLA_TABLE_CHART . "}.user_id, type, path, sharing, updated, added, {" . QMO_BUGZILLA_TABLE_MY_CHART . "}.pagenum FROM {" . QMO_BUGZILLA_TABLE_CHART . "} LEFT OUTER JOIN {" . QMO_BUGZILLA_TABLE_MY_CHART . "} ON ({" . QMO_BUGZILLA_TABLE_CHART . "}.id = {" . QMO_BUGZILLA_TABLE_MY_CHART . "}.chart_id)" . $where . $sharing_condition . $filter_query;
  //echo $query;
  $header = qmo_bugzilla_manage_header(TRUE);
  //$result = pager_query('SELECT n.*, u.name FROM {node} n '. $filter['join'] .' INNER JOIN {users} u ON n.uid = u.uid '. $filter['where'] .' ORDER BY n.changed DESC', 50, 0, NULL, $filter['args']);
  $result = pager_query($query . tablesort_sql($header), 50, 0, NULL, $query_args);
  if (!$result) {
    $reply[] = array('fail-charts');
    set_messages($reply);
    return FALSE;
  }
  else {
    $chart_ids = array();
    global $user;
    if (user_access(QMO_BUGZILLA_EDIT_ANY)) {
      $all_edit = TRUE;
    }
    while ($row = db_fetch_array($result)) {
      $chart_ids[$row['id']] = '';
      $data = qmo_bugzilla_chart_hasdata($row['id']);
      //for timeplots, show timestamped data file
      $form['title'][$row['id']] = array('#value' => $row['title']);
      if ($all_edit) {
        $edit = TRUE;
      }
      elseif (($row['user_id'] == $user->uid) && (user_access(QMO_BUGZILLA_EDIT_OWN))) {
        $edit = TRUE;
      }
      elseif (qmo_check_sharing($row['id']) && (user_access(QMO_BUGZILLA_EDIT_SHARED))) {
        $edit = TRUE;
      }
      else {
        $edit = FALSE;
      }

      $form['edit'][$row['id']] = array('#value' => $edit);
      $form['type'][$row['id']] =  array('#value' => $row['type']);
      $form['user'][$row['id']] =  array('#value' => $row['user_id']);
      $form['pagenum'][$row['id']] =  array('#value' => $row['pagenum']);
      $form['path'][$row['id']] =  array('#value' => $row['path']);
      $form['sharing'][$row['id']] =  array('#value' => $row['sharing']);
      $form['updated'][$row['id']] =  array('#value' => $row['updated']);
      $form['added'][$row['id']] =  array('#value' => $row['added']);
      $form['data'][$row['id']] =  array('#value' => $data);
    }
  }
  $form['chart_ids'] = array('#type' => 'checkboxes', '#options' => $chart_ids);
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));
  $form['#theme'] = 'form_qmo_bugzilla_charts_manage';
  $form['#method'] = 'get';
  if ($op == t('Update')) {
    $opnum = $_GET['operation'];
    $form_op['charts'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);
    $ids = $_GET['chart_ids'];
    $ids = qmo_bugzilla_filter_ids($ids, array_keys($chart_ids));
    if (empty($ids)) {
      $reply[] = array('manage-nothing');
    }
    else {
      if ($opnum == 10) {
        $ids_string = '';
        foreach ($ids as $id) {
          $ids_string .= '%d,';
        }
        $ids_string = substr($ids_string, 0, -1);
        $query = "SELECT weight, chart_id FROM {" . QMO_BUGZILLA_TABLE_MY_CHART . "} WHERE chart_id IN ( " . $ids_string . ") AND (user_id = %d) ";
        $args = $ids;
        $args[] = $user->uid;
        $result = db_query($query, $args);
        while ($row = db_fetch_object($result)) {
          $weights[$row->chart_id] = $row->weight;
        }
        $form_op['chart_weights'] = array(
          '#type' => 'fieldset',
          '#title' => t('Charts'),
          '#tree' => TRUE,
        );
        $form_op['new_charts'] = array(
          '#type' => 'hidden',
          '#tree' => TRUE,
        );
      }

      foreach ($ids as $id) {
        $chart = db_fetch_array(db_query('SELECT title FROM {' . QMO_BUGZILLA_TABLE_CHART . '} WHERE id = %d', $id));
        $title = $chart['title'];
        switch ($opnum) {
          //share
          case 10:
            $form_op['chart_weights'][$id] = array(
              '#type' => 'textfield',
              '#title' => t('W'),
              '#default_value' => get_first_nonempty($weights[$id], 0),
              '#size' => 2,
              '#maxlength' => 2,
              '#prefix' => '<li><span class="inline">',
              '#suffix' => '</span>' . check_plain($title) ."</li>\n",
            );
            if (is_null($weights[$id])) {
              $form_op['new_charts'][$id] = array(
                '#type' => 'hidden',
                '#value' => $id,
              );
            }
            break;
          default:
            $form_op['charts'][$id] = array(
              '#type' => 'hidden',
              '#value' => $id,
              '#prefix' => '<li>',
              '#suffix' => check_plain($title) ."</li>\n",
            );
        }
      }
    }
    $form_op['operation'] = array('#type' => 'hidden', '#value' => $_GET['operation']);
    //check edit access for all options but "my charts"
    //$cant = FALSE;
    switch ($opnum) {
      case 10:
      case 11:
        break;
      default:
        //qmo_bugzilla_access_chart_edit
        foreach ($ids as $id) {
          $edit = $form['edit'][$id]['#value'];
          if (!$edit) {
            drupal_access_denied();
            //$cant = TRUE;
          }
        }
        break;
    }
    switch ($opnum) {
      case -1:
      case 0:
      case 1:
        return confirm_form($form_op, t("Change type of the following charts?"), 'qmo_bugzilla/manage', t("You can change the type back at anytime."), t('Change'), t('Go back'), 'confirm-change-' . $opnum);
      case 5:
        return confirm_form($form_op, t("Empty the following charts?"), 'qmo_bugzilla/manage', NULL, t('Empty'), t('Go back'), 'confirm-empty');
      case 6:
        return confirm_form($form_op, t("Are you sure you want to delete the following charts?"), 'qmo_bugzilla/manage', NULL, t('Delete'), t('Go back'), 'confirm-delete');
        break;
      case 8:
        $form_op['sharing'] = array(
          '#type' => 'fieldset',
          '#title' => t('Sharing'),
          '#tree' => TRUE,
          '#description' => t('Share selected charts with any of the following user roles (leave blank to keep private)'),
          '#collapsible' => TRUE,
        );
        $roles = user_roles(FALSE, QMO_BUGZILLA_ACCESS);
        foreach ($roles as $rid => $role) {
          $form_op['sharing'][$rid] = array(
            '#type' => 'checkbox',
            '#title' => $role,
            '#default_value' => FALSE,
          );
        }
        return confirm_form($form_op, t("Share with selected roles..."), 'qmo_bugzilla/manage', "Share these charts with selected roles", t('Share'), t('Go back'), 'confirm-share');
        break;
      case 9:
        return confirm_form($form_op, t("Remove alias for the following charts"), 'qmo_bugzilla/manage', t("This will delete the alias for every chart."), t('Remove aliases'), t('Go back'), 'confirm-rem-alias');
        break;
      case 10:
        return confirm_form($form_op, t("Add the following to <em>My charts</em>:"), 'qmo_bugzilla/manage', t("Specify the weights for each newly added chart. For existing <em>My charts</em>, current weights have been inserted by default."), t('Add now'), t('Go back'), 'confirm-add-my');
        break;
      case 11:
        return confirm_form($form_op, t("Remove the following from <em>My charts</em>:"), 'qmo_bugzilla/manage', NULL, t('Remove now'), t('Go back'), 'confirm-remove-my');
        break;
    }
  }
  $form['filters'] = qmo_bugzilla_charts_filters();
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $options = get_value('chart_manage_options');
  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => $_GET['operation'],
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#submit' => array('form_qmo_bugzilla_charts_manage_submit'),
  );
  set_messages($reply);
  return $form;
}

function qmo_bugzilla_charts_fields() {
  global $user;
  $uid = $user->uid;
  $fields = array();
  $fields['type'] = array();
  $fields['type']['form'] = array();
  $fields['type']['title'] = t('Type');
  //first form element
  $fields['type']['form'][0]['type'] = 'select';
  $fields['type']['form'][0]['multiple'] = TRUE;
  $fields['type']['form'][0]['size'] = 3;
  $fields['type']['form'][0]['name'] = 'type';
  $fields['type']['form'][0]['options'] = array(
    -1 => get_value('type:-1'),
    0 => get_value('type:0'),
    1 => get_value('type:1'),
  );

  $fields['user_id'] = array();
  $fields['user_id']['form'] = array();
  $fields['user_id']['title'] = t('Owner');
  $result = db_query("SELECT DISTINCT user_id FROM " . QMO_BUGZILLA_TABLE_CHART);
  $res = 0;
  while ($row = db_fetch_object($result)) {
    $res = 1;
    $fields['user_id']['form'][0]['options'][$row->user_id] = user_load($row->user_id)->name;
  }
  if (!$res) {
    $fields['user_id']['form'][0]['options'] = array();
  }
  $fields['user_id']['form'][0]['type'] = 'select';
  $fields['user_id']['form'][0]['multiple'] = TRUE;
  $fields['user_id']['form'][0]['size'] = 3;
  $fields['user_id']['form'][0]['name'] = 'users';

  $roles = user_roles(FALSE, QMO_BUGZILLA_ACCESS);
  if (!$roles) {
    $roles = array();
  }
  $fields['sharing'] = array();
  $fields['sharing']['form'] = array();
  $fields['sharing']['title'] = t('Sharing');
  //first form element
  $fields['sharing']['form'][0]['type'] = 'select';
  $fields['sharing']['form'][0]['options'] = $roles;
  $fields['sharing']['form'][0]['multiple'] = TRUE;
  $fields['sharing']['form'][0]['size'] = 3;
  $fields['sharing']['form'][0]['name'] = 'sharing';

  $pagecount = variable_get('qmo_bugzilla_my_pagecount_' . $uid, 1);
  for ($i=1; $i<=$pagecount; $i++) {
    $pages[$i] = variable_get('qmo_bugzilla_my_page_' . $uid . '_' . $i . '_title', t('Page ' . $i));
  }
  $fields['pagenum'] = array();
  $fields['pagenum']['form'] = array();
  $fields['pagenum']['title'] = t('My pages');
  //first form element
  $fields['pagenum']['form'][0]['type'] = 'select';
  $fields['pagenum']['form'][0]['options'] = $pages;
  $fields['pagenum']['form'][0]['multiple'] = TRUE;
  $fields['pagenum']['form'][0]['size'] = 3;
  $fields['pagenum']['form'][0]['name'] = 'pagenum';
  
  $fields['title'] = array();
  $fields['title']['form'] = array();
  $fields['title']['title'] = t('Title');
  //first form element
  $fields['title']['form'][0]['type'] = 'select';
  $fields['title']['form'][0]['multiple'] = FALSE;
  //$fields['title']['form'][0]['size'] = 2;
  $fields['title']['form'][0]['name'] = 'title-option';
  $fields['title']['form'][0]['options'] = array(
    1 => t('contains'),
    0 => t('does not contain'),
  );
  $fields['title']['form'][0]['default'] = 1;
  //second form element
  $fields['title']['form'][1]['type'] = 'textfield';
  $fields['title']['form'][1]['size'] = 14;
  $fields['title']['form'][1]['name'] = 'title';
  
  $fields['path'] = array();
  $fields['path']['form'] = array();
  $fields['path']['title'] = t('Path');
  //first form element
  
  $fields['path']['form'] = $fields['title']['form'];
  $fields['path']['form'][0]['name'] = 'path-option';
  $fields['path']['form'][1]['name'] = 'path';
    
  return $fields;
}

function qmo_bugzilla_charts_filters() {
  $fields = qmo_bugzilla_charts_fields();
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Search'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $prefix = '<table>';
  $prefix2 = '<tr>';
  $suffix = '</table>';
  $suffix2 = '</tr>';
  foreach ($fields as $key => $field) {
    $prefix = '<table>';
    $last_key = $key;
    //make a fieldset with the whole thing:
    $form[$key] = array(
      '#type' => 'fieldset',
      '#title' => t($field['title']),
      '#prefix' => '<div class="manage-container container-' . $key . '">',
      '#suffix' => '</div>',
    );
    //for each form element put them in the fieldset
    foreach ($field['form'] as $elem) {
      $last_elem = $elem['name'];
      $form[$key][$elem['name']] = array(
        '#type' => $elem['type'],
        '#size' => $elem['size'],
        '#default_value' => get_first_nonempty($_GET[$elem['name']], $elem['default'], NULL),
        '#prefix' => $prefix . '<tr><td>',
        '#suffix' => '</td></tr>',
      );
      if ($elem['type'] == 'select') {
        $form[$key][$elem['name']]['#multiple'] = $elem['multiple'];
        $form[$key][$elem['name']]['#options'] = $elem['options'];
      }
    $prefix = "";
    }
    $form[$key][$last_elem]['#suffix'] .= $suffix;
  }
  
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Find'),
    '#prefix' => '<span class="charts-find">',
    '#suffix' => '</span>',
    //'#submit' => array('form_qmo_bugzilla_charts_manage_find'),
  );
  return $form;
}

function qmo_bugzilla_admin_validate($form, &$form_state) {
  /*
  $pagecount = $form_state['values']['qmo_bugzilla_pagecount'];
  if (!is_numeric($pagecount)) {
    form_set_error('qmo_bugzilla_pagecount', t('You must select a number for the number of pages. '));
  }
  else if ($pagecount <= 0) {
    form_set_error('qmo_bugzilla_pagecount', t('The number of pages must be positive. '));
  }
  $maintag = $form_state['values']['qmo_bugzilla_pagecount'];
  */
}

/**
 * Admin form for QMO Bugzilla
 *
 */
function qmo_bugzilla_admin($form_state) {
  $form = array();
  $form['qmo_bugzilla_chart_fullpath'] = array(
    '#type' => 'textfield',
    '#title' => t('Full path to timeplot charts (include trailing slash)'),
    '#default_value' => variable_get('qmo_bugzilla_chart_fullpath', ''),
    '#size' => 60,
    '#maxlength' => 1000,
    '#description' => t('The full path to the folder where timeplot xml and txt files are being stored. Usually "/var/www/yoursite/timeplot_data/"'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_chart_relpath'] = array(
    '#type' => 'textfield',
    '#title' => t('Relative path to timeplot charts (include trailing slash)'),
    '#default_value' => variable_get('qmo_bugzilla_chart_relpath', ''),
    '#size' => 60,
    '#maxlength' => 1000,
    '#description' => t('The relative path from the root of the domain to the folder where timeplot xml and txt files are being stored. E.g if your site is at site.com/yoursite", relpath would be "/yoursite/timeplot_data/"'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_chart_drupalpath'] = array(
    '#type' => 'textfield',
    '#title' => t('Drupal path to timeplot charts (include trailing slash)'),
    '#default_value' => variable_get('qmo_bugzilla_chart_drupalpath', ''),
    '#size' => 60,
    '#maxlength' => 1000,
    '#description' => t('The drupal path from the root of the domain to the folder where timeplot xml and txt files are being stored. E.g "timeplot_data/"'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_hours_diff'] = array(
    '#type' => 'textfield',
    '#title' => t('Hours difference between server and your location (can be negative)'),
    '#default_value' => variable_get('qmo_bugzilla_hours_diff', '0'),
    '#size' => 60,
    '#maxlength' => 60,
    '#description' => t('The difference in hours between server time and desired time (used for timeplots)'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_list_bug_url'] = array(
    '#type' => 'textfield',
    '#title' => t('List bugs URL'),
    '#default_value' => variable_get('qmo_bugzilla_list_bug_url', LIST_BUG_URL),
    '#size' => 60,
    '#maxlength' => 1000,
    '#description' => t('Show bugs by id using this url. '),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_bug_id_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Bug ID URL'),
    '#default_value' => variable_get('qmo_bugzilla_bug_id_url', BUG_ID_URL),
    '#size' => 60,
    '#maxlength' => 1000,
    '#description' => t('Show one bug by id using this url. '),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_height_default'] = array(
    '#type' => 'textfield',
    '#title' => t('Chart height'),
    '#default_value' => variable_get('qmo_bugzilla_height_default', CHART_HEIGHT_DEFAULT),
    '#size' => 60,
    '#maxlength' => 60,
    '#description' => t('Default height on chart page (can be changed for each chart)'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_dash_chart_height'] = array(
    '#type' => 'textfield',
    '#title' => t('Chart height on the dashboard'),
    '#default_value' => variable_get('qmo_bugzilla_dash_chart_height', CHART_HEIGHT_DASH_DEFAULT),
    '#size' => 60,
    '#maxlength' => 60,
    '#description' => t('All charts are the same height on the dashboard'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_block_height_default'] = array(
    '#type' => 'textfield',
    '#title' => t('Chart height in blocks'),
    '#default_value' => variable_get('qmo_bugzilla_block_height_default', CHART_BLOCK_HEIGHT_DEFAULT),
    '#size' => 60,
    '#maxlength' => 60,
    '#description' => t('Default height in blocks (can be changed for each block)'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_nontimeplot_empty'] = array(
    '#type' => 'checkbox',
    '#title' => t("No history for non-timeplot charts."),
    '#description' => t('Default: checked. If checked, old data is dropped when new data is inserted.'),
    '#default_value' => variable_get('qmo_bugzilla_nontimeplot_empty', TRUE),
    '#required' => FALSE,
  );
  return system_settings_form($form);
}

/**
 * Configure tags for QMO Bugzilla
 */
function qmo_bugzilla_admin_tags($form_state) {
  $form = array();
  $xml_tags_string = variable_get('qmo_bugzilla_xml_tags', XML_TAGS);
  if (!empty($xml_tags_string)) {
    $xml_tags = string_to_array($xml_tags_string);
    $xml_tags_count = count($xml_tags);
    $form['qmo_bugzilla_conf_xml_tags'] = array(
        '#type' => 'fieldset',
        '#title' => t('Configure tags'),
  //      '#description' => t(''),
        '#tree' => FALSE,
        '#collapsible' => TRUE,
    );
    for ($j=1; $j<=$xml_tags_count; $j++) {
      $form['qmo_bugzilla_conf_xml_tags']['xml_tag_' . $j] = array(
        '#type' => 'fieldset',
        '#title' => $xml_tags[$j-1],
        '#tree' => FALSE,
        '#collapsible' => TRUE,
      );
      $form['qmo_bugzilla_conf_xml_tags']['xml_tag_' . $j]['qmo_bugzilla_xml_tag_' . $j . '_alias'] = array(
        '#type' => 'textfield',
        '#title' => t('Alias'),
        '#default_value' => variable_get('qmo_bugzilla_xml_tag_' . $j . '_alias', $xml_tags[$j-1]),
        '#size' => 30,
        '#maxlength' => 30,
        '#description' => t('The display text for this tag, for anywhere other than on the admin pages. '),
        '#required' => FALSE,
      );
      $form['qmo_bugzilla_conf_xml_tags']['xml_tag_' . $j]['qmo_bugzilla_xml_tag_' . $j . '_separator'] = array(
        '#type' => 'textfield',
        '#title' => t('Separator'),
        '#default_value' => variable_get('qmo_bugzilla_xml_tag_' . $j . '_separator', SEPARATOR),
        '#size' => 2,
        '#maxlength' => 1,
        '#required' => FALSE,
        '#prefix' => '<div class="fltrt less-top">',
        '#suffix' => '</div>',
      );
      $form['qmo_bugzilla_conf_xml_tags']['xml_tag_' . $j]['qmo_bugzilla_xml_tag_' . $j . '_multiple'] = array(
        '#type' => 'checkbox',
        '#title' => t('Contains multiple values'),
        '#default_value' => variable_get('qmo_bugzilla_xml_tag_' . $j . '_multiple', FALSE),
        '#required' => FALSE,
      );
    }
  }
  else {
    $reply = array( array('no-tags') );
    set_messages($reply);
  }
  return system_settings_form($form);
}

/**
 * Parsing info admin page
 */
function qmo_bugzilla_admin_parse($form_state) {
  $form = array();
  $form['qmo_bugzilla_parse'] = array(
      '#type' => 'fieldset',
      '#title' => t('Parsing'),
      '#tree' => FALSE,
      '#collapsible' => TRUE,
  );
  $form['qmo_bugzilla_parse']['qmo_bugzilla_maintag'] = array(
    '#type' => 'textfield',
    '#title' => t('Main tag'),
    '#default_value' => variable_get('qmo_bugzilla_maintag', MAINTAG),
    '#size' => 60,
    '#maxlength' => 60,
    '#description' => t('Main tag. E.g. for Bugzilla, "BZ:BUG", the bug. TAG NAME <b>must be</b> in upper case!'),
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_parse']['qmo_bugzilla_xml_tags'] = array(
    '#type' => 'textarea',
    '#title' => t("Inner tags"),
    '#default_value' => variable_get('qmo_bugzilla_xml_tags', XML_TAGS),
    '#description' => t('One tag per line. TAG NAME <b>must be</b> in upper case!'),
    '#cols' => 60,
    '#rows' => 5,
    '#required' => TRUE,
  );
  $form['qmo_bugzilla_parse']['qmo_bugzilla_xml_tag_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Id tag'),
    '#default_value' => variable_get('qmo_bugzilla_xml_tag_id', XML_TAG_ID),
    '#description' => t('Which of the inner tags represents the id of the main tag?'),
    '#size' => 60,
    '#maxlength' => 60,
    '#required' => TRUE,
  );
  return system_settings_form($form);
}